#include <stdio.h>
#include <string.h>
#include <math.h>
#include "tjpgd.h"
#include "fatfs.h"
#include <stm32f429i_discovery_lcd.h>
#include "semihost.h"

#define IMG_BUF_ADDR 0xD0700000
#define LCD_WIDTH 320
#define LCD_HEIGHT 240

static FIL file;
static uint8_t *img_buf = (uint8_t *)IMG_BUF_ADDR;
static int img_buf_width, img_buf_height;

static uint8_t use_img_buf;
static int img_x_offset, img_y_offset;

/* User defined device identifier */
/*
typedef struct {
    FILE *fp;      // File pointer for input function
    BYTE *fbuf;    // Pointer to the frame buffer for output function
    UINT wfbuf;    // Width of the frame buffer [pix]
} IODEV;
*/


/*------------------------------*/
/* User defined input funciton  */
/*------------------------------*/

UINT in_func(JDEC* jd, BYTE* buff, UINT nbyte)
{
    uint32_t bytesread;
    FRESULT res;

    if(buff == NULL)
    {
        FRESULT res;
        res = f_lseek(&file, f_tell(&file) + nbyte);
        if(res != FR_OK)
            semihost_print("error seek\n");

        return nbyte;
    }

    res = f_read(&file, buff, nbyte, (void *)&bytesread);
    if(res != FR_OK)
        semihost_print_int("read error: ", res);

    return bytesread;
}


/*------------------------------*/
/* User defined output funciton */
/*------------------------------*/

UINT out_func(JDEC* jd, void* bitmap, JRECT* rect)
{
    int w = rect->right - rect->left + 1;
    int h = rect->bottom - rect->top + 1;

    char *current_pixel_data = bitmap;

    for(int i=0; i<h; i++)  //radek
    {
        for(int j=0; j<w; j++)  //pixely v radku
        {
            uint16_t x_coord = rect->left + j;
            uint16_t y_coord = rect->top + i;


            if(!use_img_buf)
            {
                uint32_t pixel_color = 0xFF000000 | (current_pixel_data[0]<<16) | (current_pixel_data[1]<<8) | (current_pixel_data[2]<<0);///
                BSP_LCD_DrawPixel_Landscape(x_coord + img_x_offset, y_coord + img_y_offset, pixel_color);
            }
            else
            {
                img_buf[(y_coord*img_buf_width + x_coord)*3] = current_pixel_data[0];
                img_buf[(y_coord*img_buf_width + x_coord)*3 + 1] = current_pixel_data[1];
                img_buf[(y_coord*img_buf_width + x_coord)*3 + 2] = current_pixel_data[2];
            }
            current_pixel_data += 3;
        }
    }
    return 1;
}

void buf_get_color_data(uint16_t x, uint16_t y, uint8_t *r, uint8_t *g, uint8_t *b)
{
    *r = img_buf[(y*img_buf_width + x)*3];
    *g = img_buf[(y*img_buf_width + x)*3 + 1];
    *b = img_buf[(y*img_buf_width + x)*3 + 2];
}


void display_buf_nearest(uint16_t width, uint16_t height)
{
    const int w1=img_buf_width;
    const int h1=img_buf_height;
    const int w2=width, h2=height;

    float scale_x = (float)w1/w2;
    float scale_y = (float)h1/h2;

    for(int y=0; y<h2; y++)
    {
        for(int x=0; x<w2; x++)
        {
            int x1 = (long)(scale_x*x);
            int y1 = (long)(scale_y*y);

            uint8_t r, g, b;
            buf_get_color_data(x1, y1, &r, &g, &b);

            uint32_t pixel_color = 0xFF000000 | (r<<16) | (g<<8) | b;

            uint16_t x_offset = (LCD_WIDTH - width)>>1;
            uint16_t y_offset = (LCD_HEIGHT - height)>>1;
            BSP_LCD_DrawPixel_Landscape(x + x_offset, y + y_offset, pixel_color);
        }
    }
}

void display_buf_bilinear(uint16_t width, uint16_t height)
{
    //inspired by http://tech-algorithm.com/articles/bilinear-image-scaling/
    const int w1=img_buf_width;
    const int h1=img_buf_height;
    const int w2=width, h2=height;

    float scale_x = (float)w1/w2;
    float scale_y = (float)h1/h2;

    for(int y=0; y<h2; y++)
    {
        for(int x=0; x<w2; x++)
        {
            float scaled_x2 = scale_x*x;
            float scaled_y2 = scale_y*y;

            int x1_left = (int)floor(scaled_x2);
            int x1_right = (int)ceil(scaled_x2);

            if(x1_right == x1_left)
                x1_right += 1.0;

            int y1_top = (int)floor(scaled_y2);
            int y1_bot = (int)ceil(scaled_y2);

            if(y1_bot == y1_top)
                y1_bot += 1.0;

            float l_horiz = scaled_x2 - x1_left;
            float l_vert = scaled_y2 - y1_top;


            uint8_t color_lt_r, color_lt_g, color_lt_b;
            uint8_t color_rt_r, color_rt_g, color_rt_b;
            uint8_t color_lb_r, color_lb_g, color_lb_b;
            uint8_t color_rb_r, color_rb_g, color_rb_b;

            buf_get_color_data(x1_left, y1_top, &color_lt_r, &color_lt_g, &color_lt_b);
            buf_get_color_data(x1_right, y1_top, &color_rt_r, &color_rt_g, &color_rt_b);
            buf_get_color_data(x1_left, y1_bot, &color_lb_r, &color_lb_g, &color_lb_b);
            buf_get_color_data(x1_right, y1_bot, &color_rb_r, &color_rb_g, &color_rb_b);

            uint8_t color_top_r = color_lt_r + l_horiz*(color_rt_r - color_lt_r);
            uint8_t color_top_g = color_lt_g + l_horiz*(color_rt_g - color_lt_g);
            uint8_t color_top_b = color_lt_b + l_horiz*(color_rt_b - color_lt_b);

            uint8_t color_bot_r = color_lb_r + l_horiz*(color_rb_r - color_lb_r);
            uint8_t color_bot_g = color_lb_g + l_horiz*(color_rb_g - color_lb_g);
            uint8_t color_bot_b = color_lb_b + l_horiz*(color_rb_b - color_lb_b);

            uint8_t color_r = color_top_r + l_vert*(color_bot_r - color_top_r);
            uint8_t color_g = color_top_g + l_vert*(color_bot_g - color_top_g);
            uint8_t color_b = color_top_b + l_vert*(color_bot_b - color_top_b);

            uint32_t color = 0xFF000000 | color_r<<16 | color_g<<8 | color_b;

            uint16_t x_offset = (LCD_WIDTH - width)>>1;
            uint16_t y_offset = (LCD_HEIGHT - height)>>1;
            BSP_LCD_DrawPixel_Landscape(x + x_offset, y + y_offset, color);
        }
    }
}

int display_jpg(const char *filename)
{
    char work[3100];  // Work area
    JDEC jdec;        // Decompression object
    JRESULT res;      // Result code of TJpgDec API
    uint8_t decomp_scale = 0;

    uint16_t final_img_w, final_img_h;


    FATFS RAMDISKFatFs;
    if(f_mount(&RAMDISKFatFs, (TCHAR const*)SDRAMDISK_Path, 0) != FR_OK)
    {
      semihost_print("ram disk mount for jpg error\n");
      return 0;
    }

    // Open a JPEG file
    if(f_open(&file, filename, FA_READ) != FR_OK)
    {
        semihost_print("cannot open jpeg file\n");
        return 0;
    }

    // Prepare to decompress
    res = jd_prepare(&jdec, in_func, work, 3100, NULL);
    if (res == JDR_OK)
    {
        if(jdec.width==LCD_WIDTH && jdec.height==LCD_HEIGHT)
        {
            use_img_buf = 0;
            img_x_offset = 0;
            img_y_offset = 0;
        }
        else if((jdec.width==LCD_WIDTH && jdec.height<LCD_HEIGHT) || (jdec.width<LCD_WIDTH && jdec.height==LCD_HEIGHT))
        {
            use_img_buf = 0;
            img_x_offset = (LCD_WIDTH - jdec.width) >> 1;
            img_y_offset = (LCD_HEIGHT - jdec.height) >> 1;

        }
        else if((jdec.width<LCD_WIDTH && jdec.height<LCD_HEIGHT) || (jdec.width>LCD_WIDTH && jdec.width<2*LCD_WIDTH)
                || (jdec.height>LCD_HEIGHT && jdec.height<2*LCD_HEIGHT))
        {
            use_img_buf = 1;
            img_buf_width = jdec.width;
            img_buf_height = jdec.height;

            float lcd_ratio = (float)LCD_WIDTH/LCD_HEIGHT;
            float img_ratio = (float)img_buf_width/img_buf_height;

            if(img_ratio > lcd_ratio)
            {
                final_img_w = LCD_WIDTH;
                final_img_h = final_img_w/img_ratio;
            }
            else
            {
                final_img_h = LCD_HEIGHT;
                final_img_w = final_img_h * img_ratio;
            }

        }
        else if(jdec.width<=LCD_WIDTH*16 && jdec.height<=LCD_HEIGHT*16)
        {
            use_img_buf = 1;
            decomp_scale = 1;

            while(decomp_scale <= 2)
            {
                if((jdec.width>>decomp_scale)<2*LCD_WIDTH && (jdec.height>>decomp_scale)<2*LCD_HEIGHT)
                    break;
                ++decomp_scale;
            }
            img_buf_width = jdec.width>>decomp_scale;
            img_buf_height = jdec.height>>decomp_scale;

            float lcd_ratio = (float)LCD_WIDTH/LCD_HEIGHT;
            float img_ratio = (float)img_buf_width/img_buf_height;

            if(img_ratio > lcd_ratio)
            {
                final_img_w = LCD_WIDTH;
                final_img_h = final_img_w/img_ratio;
            }
            else
            {
                final_img_h = LCD_HEIGHT;
                final_img_w = final_img_h * img_ratio;
            }
        }
        else
        {
            semihost_print("error - image too large\n");

            BSP_LCD_SetFont(&Font16);
            BSP_LCD_DisplayStringAt_Landscape(0, 100, "image is too large", CENTER_MODE);

            return 0;
        }

        BSP_LCD_Clear(LCD_COLOR_BLACK);

        if(use_img_buf)
        {
            BSP_LCD_SetFont(&Font16);
            BSP_LCD_DisplayStringAt_Landscape(0, 112, "jpg decompression", CENTER_MODE);
        }

        res = jd_decomp(&jdec, out_func, decomp_scale);   // Start to decompress with 1/1 scaling
        if (res != JDR_OK)
            semihost_print_int("Failed to decompress: rc=", res);


        if(use_img_buf)
        {
            BSP_LCD_Clear(LCD_COLOR_BLACK);
            BSP_LCD_DisplayStringAt_Landscape(0, 100, "image resizing", CENTER_MODE);
            //display_buf_nearest(final_img_w, final_img_h);
            display_buf_bilinear(final_img_w, final_img_h);
        }

        //image info
        BSP_LCD_SetFont(&Font12);

        //image info - file name and resolution
        char str[50];
        sprintf(str, "%s  %dX%d", filename, jdec.width, jdec.height);

        //image info - file size
        int file_size = f_size(&file);
        uint16_t file_size_kb_integral, file_size_kb_decimal;

        file_size_kb_integral = file_size/1000;
        file_size_kb_decimal = file_size/100 - file_size_kb_integral*10;
        sprintf(str, "%s %d.%dKB",str, file_size_kb_integral, file_size_kb_decimal);

        BSP_LCD_DisplayStringAt_Landscape(0, 227, str, RIGHT_MODE);

    } else {
        semihost_print_int("Failed to prepare: rc=", res);
    }

    f_close(&file);

    return 1;
}

